using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.IO;
using System.Drawing.Imaging;

// Glyph image at: http://futurama.wikia.com/wiki/Alienese

namespace howto_separate_glyphs
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        // The image containing the glyphs.
        private Bitmap GlyphsBm = null;

        // The glyph position and dimensions.
        private Rectangle GlyphRect = new Rectangle(0, 0, 60, 60);

        // Initialize.
        private void Form1_Load(object sender, EventArgs e)
        {
            KeyPreview = true;
            picGlyphs.SizeMode = PictureBoxSizeMode.Normal;
        }

        // Open a glyph file.
        private void mnuFileOpen_Click(object sender, EventArgs e)
        {
            Cursor = Cursors.WaitCursor;
            Refresh();

            if (ofdFile.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    GlyphsBm = LoadBitmapUnlocked(ofdFile.FileName);
                    picGlyphs.ClientSize = GlyphsBm.Size;
                    picGlyphs.Visible = true;
                    mnuFileSaveGlyph.Enabled = true;
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }
            }
            Cursor = Cursors.Default;
        }

        // Save a glyph.
        private void mnuFileSaveGlyph_Click(object sender, EventArgs e)
        {
            if (sfdGlyph.ShowDialog() == DialogResult.OK)
            {
                using (Bitmap bm = new Bitmap(GlyphRect.Width, GlyphRect.Height))
                {
                    using (Graphics gr = Graphics.FromImage(bm))
                    {
                        Rectangle dest_rect = new Rectangle(
                            0, 0, GlyphRect.Width, GlyphRect.Height);
                        gr.DrawImage(GlyphsBm, dest_rect, GlyphRect, GraphicsUnit.Pixel);
                    }
                    SaveImage(bm, sfdGlyph.FileName);
                }
            }
        }

        // Close the application.
        private void mnuFileExit_Click(object sender, EventArgs e)
        {
            Close();
        }

        // Let the user select the glyph dimensions.
        private void mnuDataDimensions_Click(object sender, EventArgs e)
        {
            DimensionsDialog dlg = new DimensionsDialog();
            dlg.txtDimensions.Text = GlyphRect.Width.ToString() +
                " x " + GlyphRect.Height.ToString();
            if (dlg.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    string[] fields = dlg.txtDimensions.Text.ToLower().Split('x');
                    int width = int.Parse(fields[0]);
                    int height = int.Parse(fields[1]);
                    GlyphRect = new Rectangle(GlyphRect.X, GlyphRect.Y, width, height);
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }
                picGlyphs.Refresh();
            }
        }

        // Draw the glyphs and glyph rectangle.
        private void picGlyphs_Paint(object sender, PaintEventArgs e)
        {
            if (GlyphsBm == null) return;

            e.Graphics.DrawImage(GlyphsBm, 0, 0);
            using (Pen pen = new Pen(Color.Red, 3))
            {
                e.Graphics.DrawRectangle(pen, GlyphRect);
            }
        }

        // Load a bitmap without locking it.
        private Bitmap LoadBitmapUnlocked(string file_name)
        {
            using (Bitmap bm = new Bitmap(file_name))
            {
                return new Bitmap(bm);
            }
        }

        // Save the file with the appropriate format.
        public void SaveImage(Image image, string filename)
        {
            string extension = Path.GetExtension(filename);
            switch (extension.ToLower())
            {
                case ".bmp":
                    image.Save(filename, ImageFormat.Bmp);
                    break;
                case ".exif":
                    image.Save(filename, ImageFormat.Exif);
                    break;
                case ".gif":
                    image.Save(filename, ImageFormat.Gif);
                    break;
                case ".jpg":
                case ".jpeg":
                    image.Save(filename, ImageFormat.Jpeg);
                    break;
                case ".png":
                    image.Save(filename, ImageFormat.Png);
                    break;
                case ".tif":
                case ".tiff":
                    image.Save(filename, ImageFormat.Tiff);
                    break;
                default:
                    throw new NotSupportedException(
                        "Unknown file extension " + extension);
            }
        }

        // Mouse events.
        private bool Dragging = false;
        private Point DragOffset;
        private void picGlyphs_MouseDown(object sender, MouseEventArgs e)
        {
            // See if the mouse is inside the current GlyphRect.
            if (!GlyphRect.Contains(e.Location)) return;

            Dragging = true;
            DragOffset = new Point(
                GlyphRect.X - e.Location.X,
                GlyphRect.Y - e.Location.Y);
        }

        private void picGlyphs_MouseMove(object sender, MouseEventArgs e)
        {
            if (!Dragging) return;
            int x = e.Location.X + DragOffset.X;
            int y = e.Location.Y + DragOffset.Y;
            GlyphRect = new Rectangle(x, y, GlyphRect.Width, GlyphRect.Height);
            picGlyphs.Refresh();
        }

        private void picGlyphs_MouseUp(object sender, MouseEventArgs e)
        {
            Dragging = false;
        }

        // Handle arrow keys.
        private void Form1_KeyDown(object sender, KeyEventArgs e)
        {
            int distance = 1;
            if (e.Shift) distance = 5;

            switch (e.KeyCode)
            {
                case Keys.Left:
                    GlyphRect.X -= distance;
                    break;
                case Keys.Right:
                    GlyphRect.X += distance;
                    break;
                case Keys.Up:
                    GlyphRect.Y -= distance;
                    break;
                case Keys.Down:
                    GlyphRect.Y += distance;
                    break;
            }
            picGlyphs.Refresh();
        }
    }
}
